//***********************************************************************************************************************
//** 																											   	   **
//**	Projekt:	LCD - Libary																					   **
//** 																											   	   **
//**	Beschreibung:																								   **
//** 	Routinen zur Behandlung und Verwaltung eines vierzeiligen LCD-Displays am AVR - Controller.  				   **
//** 																												   **
//** 																												   **
//** 	Pinbelegung:																								   **
//** 																												   **
//**	Pin [Display]	Funktion [Display]	Pin [Port]																   **
//**																												   **
//** 	4				RS					P0																		   **
//** 	5				R/W					P1																		   **
//** 	6				E1					P2																		   **
//** 	7				E2					P3																		   **
//** 	12 - 15			Data 4 - 7			P4 - P7																	   **
//** 																												   **
//** 	CPU Typ:   		AVR - Serie																					   **
//** 																								  			       **
//**	CPU-Frequenz:	max. 8 MHz																					   **
//**																												   **
//** 	Funktionen:																									   **
//** 																									   			   **
//** 	void lcd_control(unsigned char data)						-	bermittlung eines Instruktsbyte			   **
//** 																										   		   **
//**	void lcd_data(unsigned char data)							-	bermittlung eines Datenbyte				   **
//** 																									  			   **
//**	void lcd_init(void)											-	Initialisierung								   **
//** 																									   			   **
//** 	void lcd_position (unsigned int Zeile, unsigned int Spalte)	-	Position des Cursors			   			   **
//** 																									   			   **
//** 	void lcd_clear(void)										-	Leeren der Anzeige des Displays	   			   **
//** 																									   			   ** 
//***********************************************************************************************************************

#ifndef F_CPU

	#define F_CPU 8000000UL				//CPU - Frequenz wird eingestellt

#endif

#define LCD_PORT PORTD					//Festlegen des Ports, an dem dass Display angeschloen ist

#define RS 0x01
#define RW 0x02
#define E1 0x04
#define E2 0x08

#include <util/delay.h>					//Einbinden der Datei 'util/delay.h'
#include <avr/io.h>						//Einbinden der Datei 'avr/io.h'

unsigned char XPosition =1;				//'XPosition' wird als ganzahlige Variable eingefhrt	
unsigned char YPosition =1;				//'YPosition' wird als ganzahlige Variable eingefhrt	

//***********************************************************************************************************************
//																													   **
//   Funktion 'lcd_control' zum Senden von Befehlen wird eingestellt. Dabei wird der Funktion das Parameter 'data' 	   **
//   und 'enable' bergeben.																						   **
//   																												   **
//***********************************************************************************************************************

void lcd_control(unsigned char data, unsigned char enable)		
{

  unsigned char Port = 0x00;			//'Port' wird als ganzahlige Variable eingefhrt		

  Port 		= (data & 0xF0);   			//Ausmaskieren der unteren 4 Bit

  LCD_PORT 	= Port;              		//Ausgabe der Daten auf 'LCD_PORT' und RS = '0'
  LCD_PORT 	= Port | enable;       		//Enable E 	= 	'1' setzen

  asm("nop");                   		//Warteschleife von min. 220 ns
  asm("nop");
  asm("nop");	

  LCD_PORT 	= Port;              		//Enable E 	= 	'0' setzen
		
  Port 		= (data & 0x0F) << 4;	   	//Ausmaskieren der oberen 4 Bit und verschieben auf Pin4 - Pin 7
    												
  LCD_PORT 	= Port;     							         	
  LCD_PORT 	= Port | enable;       		//Enable E 	= 	'1' setzen

  asm("nop");                   		//Warteschleife von min. 220 ns
  asm("nop");
  asm("nop");

  LCD_PORT 	= Port;              		//Enable E 	= 	'0' setzen

  _delay_us(50);						//Warteschleife von min. 42 us
	
}

//***********************************************************************************************************************
//																													   **
//   Funktion 'lcd_position' zum Setzen der Cursorposition wird eingestellt. Dabei wird der Funktion das Parameter 	   **
//   'Zeile', 'enable' und 'Spalte' bergeben.																		   **
//   																												   **
//***********************************************************************************************************************

void lcd_position (unsigned int Zeile, unsigned int Spalte)
{
	if(YPosition <= 2)
	{

		lcd_control (0x80 + Spalte - 1 + 0x40*(Zeile - 1), E1);	

	}
	else
	{

		lcd_control (0x80 + Spalte - 1 + 0x40*(Zeile - 2 - 1), E2);	

	}

	XPosition = Spalte;					//XPosition' 	= 	'Spalte' 
	YPosition = Zeile;					//'YPosition'  	= 	'Zeile' 
}

//***********************************************************************************************************************
//																													   **
//   Funktion 'lcd_data' zum Senden von Zeichen wird eingestellt. Dabei wird der Funktion das Parameter 'data' und 	   **
//   'enable'bergeben.																		   						   **
//   																												   **
//***********************************************************************************************************************

void lcd_data(unsigned char data, unsigned char enable)
{

  	unsigned char Port = 0x00;			//'Port' wird als ganzahlige Variable eingefhrt		

 	Port 		= (data & 0xF0);   		//Ausmaskieren der unteren 4 Bit
	Port		= Port | RS;			//RS = '1'

 	LCD_PORT 	= Port;              	//Ausgabe der Daten auf 'LCD_PORT' und RS = '0'
  	LCD_PORT 	= Port | enable;       	//Enable E 	= 	'1' setzen

 	asm("nop");                   		//Warteschleife von min. 220 ns
  	asm("nop");
 	asm("nop");	

  	LCD_PORT 	= Port;              	//Enable E 	= 	'0' setzen
		
  	Port 		= (data & 0x0F) << 4;	//Ausmaskieren der oberen 4 Bit und verschieben auf 
										//Pin4 - Pin7

	Port		= Port | RS;			//RS = '1'
    											
  	LCD_PORT 	= Port;     							         	
  	LCD_PORT 	= Port | enable;       	//Enable E 	= 	'1' setzen

  	asm("nop");                   		//Warteschleife von min. 220 ns
  	asm("nop");	
  	asm("nop");

  	LCD_PORT 	= Port;              	//Enable E 	= 	'0' setzen

  	_delay_us(50);						//Warteschleife von min. 42 us


	XPosition = XPosition + 1;		
							
  	if(XPosition == 28)					//Wenn 'XPosition' den Wert 28 erreicht hat, dann ...
  	{

    	XPosition = 1;					//... setze 'XPosition' zurck auf den Wert '1',
		YPosition = YPosition + 1;		//... YPosition = YPosition + 1,

		if(YPosition == 5)				//... wenn 'YPosition' den Wert 5 erreicht hat, dann ...
		{

			YPosition = 1;				//... setze 'YPosition' zurck auf den Wert '1',

		}
  }

  lcd_position(YPosition, XPosition);	//Der Funktion 'lcd-position' werden Parameter bergeben
}

//***********************************************************************************************************************
//																													   **
//   Funktion 'lcd_clear' zum Lschen des LCD - Displays wird eingestellt.											   **
//   																												   **
//***********************************************************************************************************************

void lcd_clear ()				
{	
	
	lcd_control(0x01, E1);         		//Display wird 'geleert' 
	lcd_control(0x01, E2);      		//Display wird 'geleert'

	_delay_ms (2);						//Warteschleife von min. 1,64 ms

	XPosition = 1;						//'XPosition' wird auf den Wert '1' gesetzt
	YPosition= 1; 						//'Yosition' wird auf den Wert '1' gesetzt
}

//***********************************************************************************************************************
//																													   **
//   Funktion 'lcd_init' zum Initialisieren des LCD - Displays wird eingestellt.									   **
//   																												   **
//***********************************************************************************************************************

void lcd_init (void)
{
  _delay_ms(17);              			//Display muss min. 15 ms warten, nachdem die Betriebsspannung angelegt wurde
  lcd_control(0x28, E1);           		//System set:	N = 1 - vierzeiliges Display, 4 - Bit - Modus

  _delay_ms(5);                			//Warteschleife von min. 4,1 ms 
  lcd_control(0x28, E1);                	

  _delay_us(120);                		//Warteschleife von min. 100 us
  lcd_control(0x28, E1);                	

  _delay_us(100);						//Warteschleife von etwa 100 us - Busy flag 

  lcd_control(0x0C, E1);          		//Display wird eingeschaltet, Cursor wird ausgeschaltet

  lcd_control(0x06, E1);        		//Schreibrichtung wird festgelegt 

  lcd_control(0x02, E1);           		//Platzierung des Cursors an Position '0'
  _delay_ms (2);						//Warteschleife von min. 1,64 ms
	
									

   _delay_ms(17);              			//Display muss min. 15 ms warten, nachdem die Betriebsspannung angelegt wurde
  lcd_control(0x28, E2);          		//System set:	N = 1 - vierzeiliges Display, 4 - Bit - Modus

  _delay_ms(5);                			//Warteschleife von min. 4,1 ms 
  lcd_control(0x28, E2);                	

  _delay_us(120);                		//Warteschleife von min. 100 us
  lcd_control(0x28, E2);                	

  _delay_us(100);						//Warteschleife von etwa 100 us - Busy flag 

  lcd_control(0x0C, E2);           		//Display wird eingeschaltet, Cursor wird ausgeschaltet

  lcd_clear();                  		//Display wird gelscht

  lcd_control(0x06, E2);          		//Schreibrichtung wird festgelegt 

  lcd_control(0x02, E2);        		//Platzierung des Cursors an Position '0'
  _delay_ms (2);						//Warteschleife von min. 1,64 ms

}

//***********************************************************************************************************************
//																													   **
//   Funktion 'lcd_text' zum Ausgeben eines Textes wird eingestellt. Dabei wird der Funktion das Parameter 'text'	   **
//   bergeben.																										   **
//																													   **
//***********************************************************************************************************************

void lcd_text (char *text)
{
	unsigned int i;						//'i' wird als ganzahlige Variable eingefhrt	

	i = 0;								//'i' wird der Wert '0' bergeben

	while (text[i] != 0)				//Schleife 
	{
		if(YPosition <= 2)
		{

			lcd_data (text[i], E1);		

		}
		else
		{

			lcd_data (text[i], E2);

		}

		i = i + 1;					
	}
}






